'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
class MenuService extends Service {
  async create({ name, code, state, type, uri, en }) {
    assert(name, '名称不存在');
    assert(code, '编码不存在');
    assert(state, '状态不存在');
    assert(type, '类型不存在');
    assert(en, '英文缩写不存在');
    const { Menu: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      const isdata = await model.find({ code });
      if (isdata.length > 0) return { errmsg: '编码已存在', errcode: -2001 };
      await model.create({ name, code, createAt, state, type, uri, en });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '添加失败' });
    }
  }
  async update({ name, state, _id, uri, en }) {
    assert(_id, 'id不存在');
    const { Menu: model } = this.ctx.model;
    try {
      await model.findById(_id).update({ name, state, uri, en });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '修改失败' });
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Menu: model } = this.ctx.model;
    const { Role: Rolemodel } = this.ctx.model;
    const { Page: Pagemodel } = this.ctx.model;
    const { Column: Columnmodel } = this.ctx.model;
    try {
      const menu = await model.findById(id);
      const roleList = await Rolemodel.where('adminMenuList').in(menu.code);
      roleList.forEach(async e => {
        const adminMenuList = e.adminMenuList.filter(p => p !== menu.code);
        await Rolemodel.findById(e._id).update({ adminMenuList });
      });
      switch (menu.type) {
        case 0: {
          // 栏目
          const menuList = await Columnmodel.find({ menuList: { $in: [ menu.code ] } });
          menuList.filter(async p => {
            const menuList = p.menuList.filter(p => p !== menu.code);
            await Columnmodel.findById(p._id).update({ menuList });
          });
          break;
        }
        case 2: {
          // 单页
          const pageList = await Pagemodel.find({ menu: menu.code });
          pageList.filter(async p => {
            await Pagemodel.findById(p._id).update({ menu: null });
          });
          break;
        }
        case 3: {
          // 父级
          const pmenu = await model.find({ pid: menu.code });
          if (pmenu.length > 0) {
            return { errmsg: '存在子级菜单，请先删除子级菜单', errcode: -1001 };
          }
          break;
        }
        default: {
          break;
        }
      }
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '删除失败' });
    }
  }
  async query({ skip, limit, name, code, type }) {
    const { Menu: model } = this.ctx.model;
    try {
      const filter = {};
      if (name) filter.name = name;
      if (code) filter.code = code;
      if (type) filter.type = type;
      const total = await model.find({ ...filter });
      let res;
      if (skip !== undefined && limit !== undefined) {
        res = await model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await model.find({ ...filter });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '查询失败' });
    }
  }
}

module.exports = MenuService;
