'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
class PageService extends Service {
  async create({ title, annex, content, istop, menu, annexname, date }) {
    assert(title, '标题不存在');
    assert(content, '内容不存在');
    assert(content, '内容不存在');
    const { Page: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      let text = content.replace(new RegExp('</?[^>]+>', 'gm'), '');
      text = text.replace('\\s*|\t|\r|\n', '');
      const slug = text.substring(0, 200);
      const menus = await model.find({ menu });
      if (menus.length > 0) return { errmsg: '不能重复绑定菜单', errcode: -2001 };
      await model.create({ title, slug, date, annex, content, istop, menu, createAt, annexname });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '添加失败' });
    }
  }
  async update({ title, annex, content, istop, menu, _id, annexname, date }) {
    assert(_id, 'id不存在');
    const { Page: model } = this.ctx.model;
    try {
      let text = content.replace(new RegExp('</?[^>]+>', 'gm'), '');
      text = text.replace('\\s*|\t|\r|\n', '');
      const slug = text.substring(0, 200);
      const menus = await model.find({ menu });
      // eslint-disable-next-line eqeqeq
      if (menus.length > 0 && menus[0]._id != _id) return { errmsg: '不能重复绑定菜单', errcode: -2001 };
      await model.findById(_id).update({ title, date, slug, annex, content, istop, menu, annexname });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '修改失败' });
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Page: model } = this.ctx.model;
    try {
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '删除失败' });
    }
  }
  async details({ id }) {
    assert(id, 'id不存在');
    const { Page: model } = this.ctx.model;
    try {
      const res = await model.findById(id);
      return { errmsg: '', errcode: 0, data: res };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '删除失败' });
    }
  }
  async detailscode({ code }) {
    assert(code, 'code不存在');
    const { Page: model } = this.ctx.model;
    try {
      const res = await model.find({ menu: code });
      return { errmsg: '', errcode: 0, data: res };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '删除失败' });
    }
  }
  async query({ skip, limit, title }) {
    const { Page: model } = this.ctx.model;
    const filter = {};
    if (title) filter.title = title;
    try {
      const total = await model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await model.find({ ...filter }, { content: false }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await model.find({ ...filter }, { content: false });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '查询失败' });
    }
  }
}

module.exports = PageService;
